package cz.muni.pa165.genre.rest;

import cz.muni.pa165.core.api.genre.GenreDto;
import cz.muni.pa165.genre.facade.GenreFacade;
import io.swagger.v3.oas.annotations.Operation;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

/**
 * Spring REST Controller for communication using HTTP protocol and JSON data format.
 */
@RestController
@RequestMapping(path = "${rest.path.genres}")
@Slf4j
public class GenreRestController {

    private final GenreFacade genreFacade;

    @Autowired
    GenreRestController(GenreFacade genreFacade) {
        this.genreFacade = genreFacade;
    }

    @Operation(
        summary = "Get all genres",
        description = """
                    Returns an array of all genres. The array can be paginated using the 'page' and 'size' query parameters.
                    """
    )
    @GetMapping
    @CrossOrigin(origins = "*")
    public Page<GenreDto> getAllGenres(
        @ParameterObject Pageable pageable,
        HttpServletRequest req
    ) {
        log.info(
            "{} {} called from {}",
            req.getMethod(),
            req.getRequestURI(),
            req.getRemoteHost()
        );

        return genreFacade.findAll(pageable);
    }

    @Operation(
        summary = "Get a specific genre by its id",
        description = """
                    Returns a genre with the given id. If the genre does not exist, the server returns a 404 status code.
                    """
    )
    @GetMapping(path = "${rest.path.id}")
    @CrossOrigin(origins = "*")
    public GenreDto getGenre(@PathVariable Long id, HttpServletRequest req) {
        log.info(
            "{} {} called from {}",
            req.getMethod(),
            req.getRequestURI(),
            req.getRemoteHost()
        );

        return genreFacade.findById(id);
    }

    @Operation(
        summary = "Post a new genre",
        description = """
                    Creates a new genre with the given name. If the name is empty, the server returns a 400 status code.
                    """
    )
    @PostMapping
    @CrossOrigin(origins = "*")
    @ResponseStatus(HttpStatus.CREATED)
    public GenreDto createGenre(
        @Valid @RequestBody GenreDto newGenre,
        BindingResult bindingResult,
        HttpServletRequest req,
        @RequestHeader(value = "User-agent", required = false) String userAgent
    ) {
        log.info(
            "{} {} called from {} by {}",
            req.getMethod(),
            req.getRequestURI(),
            req.getRemoteHost(),
            userAgent
        );

        if (bindingResult.hasErrors()) {
            log.debug("Binding errors found");

            for (FieldError fe : bindingResult.getFieldErrors()) {
                log.debug(
                    "FieldError: {} - {}",
                    fe.getField(),
                    fe.getDefaultMessage()
                );
                throw new ResponseStatusException(
                    HttpStatus.BAD_REQUEST,
                    fe.getField() + " - " + fe.getDefaultMessage()
                );
            }
        }

        return genreFacade.create(newGenre);
    }

    @Operation(
        summary = "Delete a genre by ID",
        description = """
                    Deletes genre with a given ID, if it exists. If the genre does not exist, the server returns a 404 status code.
                    """
    )
    @DeleteMapping(path = "${rest.path.id}")
    @CrossOrigin(origins = "*")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public void deleteGenre(@PathVariable Long id, HttpServletRequest req) {
        log.info(
            "{} {} called from {}",
            req.getMethod(),
            req.getRequestURI(),
            req.getRemoteHost()
        );
        genreFacade.delete(id);
    }

    @Operation(
        summary = "Update a genre by ID",
        description = """
                    Updates genre with a given ID, if it exists. If the genre does not exist, the server returns a 404 status code.
                    """
    )
    @PutMapping(path = "${rest.path.id}")
    @CrossOrigin(origins = "*")
    public GenreDto updateGenre(
        @PathVariable Long id,
        @RequestBody GenreDto genre,
        HttpServletRequest req
    ) {
        log.info(
            "{} {} called from {}",
            req.getMethod(),
            req.getRequestURI(),
            req.getRemoteHost()
        );

        genre.setId(id);
        return genreFacade.update(genre);
    }
}
