package cz.muni.pa165.movie.rest;

import cz.muni.pa165.core.api.movie.MovieBasicViewDto;
import cz.muni.pa165.core.api.movie.MovieDetailedViewDto;
import cz.muni.pa165.core.api.rating.MovieRatingDto;
import cz.muni.pa165.movie.data.repository.MovieFilter;
import cz.muni.pa165.movie.facade.MovieFacade;
import io.swagger.v3.oas.annotations.Operation;
import jakarta.validation.Valid;
import java.util.List;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping(path = "${rest.path.movies}")
public class MovieRestController {

    private final MovieFacade movieFacade;

    @Autowired
    public MovieRestController(MovieFacade movieFacade) {
        this.movieFacade = movieFacade;
    }

    @Operation(summary = "Get all movies")
    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    public Page<MovieBasicViewDto> getMovies(
        @ParameterObject Pageable pageable,
        @ParameterObject MovieFilter movieFilter
    ) {
        return movieFacade.findAll(pageable, movieFilter);
    }

    @Operation(summary = "Get movie by id")
    @GetMapping(path = "${rest.path.id}")
    @ResponseStatus(HttpStatus.OK)
    public MovieDetailedViewDto getMovieById(@PathVariable long id) {
        return movieFacade.findById(id);
    }

    @Operation(summary = "Create new movie")
    @PostMapping
    @ResponseStatus(HttpStatus.CREATED)
    public MovieDetailedViewDto createMovie(
        @Valid @RequestBody MovieDetailedViewDto movie
    ) {
        return movieFacade.create(movie);
    }

    @Operation(summary = "Update movie")
    @PutMapping(path = "${rest.path.id}")
    @ResponseStatus(HttpStatus.OK)
    public MovieDetailedViewDto updateMovie(
        @PathVariable long id,
        @RequestBody MovieDetailedViewDto movie
    ) {
        return movieFacade.update(id, movie);
    }

    @Operation(summary = "Delete movie")
    @DeleteMapping(path = "${rest.path.id}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public void deleteMovie(@PathVariable long id) {
        movieFacade.delete(id);
    }

    @Operation(summary = "Create new movie rating")
    @PostMapping(path = "${rest.path.id}/ratings")
    @ResponseStatus(HttpStatus.CREATED)
    public MovieRatingDto createMovieRating(
        @PathVariable long id,
        @RequestBody MovieRatingDto movieRating
    ) {
        movieRating.setMovieId(id);
        throw new UnsupportedOperationException("Not implemented yet");
    }

    @Operation(summary = "Get recommendations")
    @GetMapping(path = "${rest.path.id}/recommendations")
    @ResponseStatus(HttpStatus.OK)
    public List<MovieBasicViewDto> getMovieRecommendations(
        @PathVariable long id
    ) {
        return movieFacade.recommend(id);
    }
}
